Struct ReadEventsGuard

pub struct ReadEventsGuard { /* private fields */ }
Available on crate feature wayland and Linux only.
Guard for synchronizing event reading across multiple threads

If multiple threads need to read events from the Wayland socket concurrently, it is necessary to synchronize their access. Failing to do so may cause some of the threads to not be notified of new events, and sleep much longer than appropriate.

This guard is provided to ensure the proper synchronization is done. The guard is created using the Backend::prepare_read() method. And the event reading is triggered by consuming the guard using the ReadEventsGuard::read() method, synchronizing with other threads as necessary so that only one of the threads will actually perform the socket read.

If you plan to poll the Wayland socket for readiness, the file descriptor can be retrieved via the ReadEventsGuard::connection_fd() method. Note that for the synchronization to correctly occur, you must always create the ReadEventsGuard before polling the socket.

Dropping the guard is valid and will cancel the prepared read.

Implementations



impl ReadEventsGuard

pub fn connection_fd(&self) -> BorrowedFd<'_>

Access the Wayland socket FD for polling

pub fn read(self) -> Result<usize, WaylandError>

Attempt to read events from the Wayland socket

If multiple threads have a live reading guard, this method will block until all of them are either dropped or have their read() method invoked, at which point one of the threads will read events from the socket and invoke the callbacks for the received events. All threads will then resume their execution.

This returns the number of dispatched events, or 0 if an other thread handled the dispatching. If no events are available to read from the socket, this returns a [WouldBlock] IO error.



impl ReadEventsGuard

pub fn read_without_dispatch(self) -> Result<(), WaylandError>

The same as read, but doesn’t dispatch events.

To dispatch them, use dispatch_inner_queue.